import os

# import textwrap

"""
File PATHs
"""


class PATH(object):
    OPENI_FOLDER = os.path.join(
        os.path.expanduser("~"), ".openi"
    )  # os.path.expanduser('~'), os.getcwd()
    TOKEN_PATH = os.path.join(OPENI_FOLDER, "token.json")
    SAVE_PATH = os.path.join(os.getcwd(), "dataset")
    LOG_PATH = os.path.join(OPENI_FOLDER, "openi.log")
    TOKEN_PATTERN = r"^[0-9a-fA-F]{40}$"


"""
API
"""


class API(object):
    ENDPOINT = "https://openi.pcl.ac.cn"
    VERSION = "/api/v1"


"""
Datasets
"""


class DATASET(object):
    SOTRAGE_TYPE = {"GPU": 0, "NPU": 1}
    MAX_FILE_SIZE_GB = 200
    SMALL_FILE_CHUNK_SIZE = 1024 * 1024 * 8
    LARGE_FILE_CHUNK_SIZE = 1024 * 1024 * 64

    MAX_CHUNKS = 10000
    MAX_FILE_SIZE = 1024 * 1024 * 1024 * MAX_FILE_SIZE_GB
    SMALL_FILE_LIMIT = 1024 * 1024 * 8 * MAX_CHUNKS


"""
CLI help 
"""


class CLI(object):
    # main help page
    banner = "OpenI command line tool 启智AI协作平台命令行工具"

    usage = "openi {login, whoami, dataset, ...} [<args>] [-h]"
    command_login = "使用令牌登录启智并保存到本机"
    command_logout = "登出当前用户并删除本地令牌文件"
    command_whoami = "查询当前登录用户"
    command_dataset = "{upload,download} 上传/下载启智AI协作平台的数据集 "

    # Login
    login_usage = "openi login [-t token] [-e endpoint] [-h]"
    param_token = (
        "选填: 替代命令行输入，启智账号令牌，https://openi.pcl.ac.cn/user/settings/applications"
    )
    param_endpoint = "选填: 仅内部使用"

    # Dataset
    dataset_choices = ["upload", "download"]
    dataset_usage = "openi dataset {upload,download} [<args>] [-h]"

    command_dataset_upload = "上传数据集文件，需指定文件名,仓库路径及存储类型"
    dataset_upload_help = "上传数据集文件, openi dataset upload -h 查看更多说明"
    dataset_upload_usage = (
        "openi dataset upload [-f file] [-r repo_id] [-c cluster] [-h]"
    )
    dataset_upload_param_file = "本地文件名称，包含文件路径"
    dataset_upload_param_repo_id = "所在仓库路径，格式为`拥有者/仓库名`，登录用户需要拥有此仓库权限"
    dataset_upload_param_cluster = "选填: 文件的存储集群，不区分大小写，默认为`NPU`"
    # dataset_upload_epilog = textwrap.dedent(
    #     """
    #     用法说明：\n
    #     登录用户为user1，并且为user2/repo2仓库的协作者 \n
    #     用法一: openi dataset upload data1.zip user1/repo1 gpu
    #     上传本地文件`data1.zip`到repo1仓库数据集，存储类型为GPU \n
    #     用法二: openi d upload localDir/data2.zip user2/repo2
    #     上传本地文件`./localDir/data2.zip`到协作仓库user2/repo2，存储类型为NPU \n
    #     """)

    command_dataset_download = "下载数据集文件，需指定文件名,仓库路径及存储类型"
    dataset_download_help = "下载数据集文件, openi dataset download -h 查看更多说明"
    dataset_download_usage = (
        "openi dataset download "
        "[-f file] [-r repo_id] "
        "[-c cluster] [-p save_path] [-h]"
    )
    dataset_download_param_file = "网页端数据集文件名称，只能下载`.zip`或`.tar.gz`格式的文件"
    dataset_download_param_save_path = "选填: 本地的保存路径，默认为在当前路径下创建`dataset`目录"
    # dataset_download_epilog = textwrap.dedent(
    #     """
    #     用法说明： \n
    #     登录用户为user1，并且为user2/repo2仓库的协作者\n
    #     用法一: openi dataset download data1.zip user1/repo1 gpu -p /downloads
    #     从repo1仓库下载`data1.zip`(GPU)数据集文件，并存到本地`/downloads/data1.zip`\n
    #     用法二: openi d download data2.zip user2/repo2
    #     从协作仓库user2/repo2下载`data2.zip`(NPU)数据集文件，并存到本地`./dataset/data2.zip`\n
    #     """)

    param_default = "TODO"



import os
import json
import moxing as mox
from mindspore.train.callback import Callback


def openi_dataset_to_env(data_url, data_dir):
    """
    openi copy single dataset to training image
    """
    try:
        mox.file.copy_parallel(data_url, data_dir)
        print("Successfully Download {} to {}".format(data_url, data_dir))
    except Exception as e:
        print("moxing download {} to {} failed: ".format(data_url, data_dir) + str(e))
    return


def openi_multidataset_to_env(multi_data_url, data_dir):
    """
    copy single or multi dataset to training image
    """
    multi_data_json = json.loads(multi_data_url)
    for i in range(len(multi_data_json)):
        path = data_dir + "/" + multi_data_json[i]["dataset_name"]
        if not os.path.exists(path):
            os.makedirs(path)
        try:
            mox.file.copy_parallel(multi_data_json[i]["dataset_url"], path)
            print(
                "Successfully Download {} to {}".format(
                    multi_data_json[i]["dataset_url"], path
                )
            )
        except Exception as e:
            print(
                "moxing download {} to {} failed: ".format(
                    multi_data_json[i]["dataset_url"], path
                )
                + str(e)
            )
    return


def pretrain_to_env(pretrain_url, pretrain_dir):
    """
    copy pretrain to training image
    """
    pretrain_url_json = json.loads(pretrain_url)
    print("pretrain_url_json:", pretrain_url_json)
    for i in range(len(pretrain_url_json)):
        modelfile_path = pretrain_dir + "/" + pretrain_url_json[i]["model_name"]
        try:
            mox.file.copy(pretrain_url_json[i]["model_url"], modelfile_path)
            print(
                "Successfully Download {} to {}".format(
                    pretrain_url_json[i]["model_url"], modelfile_path
                )
            )
        except Exception as e:
            print(
                "moxing download {} to {} failed: ".format(
                    pretrain_url_json[i]["model_url"], modelfile_path
                )
                + str(e)
            )
    return


def env_to_openi(train_dir, train_url):
    """
    upload output to openi
    """
    device_num = int(os.getenv("RANK_SIZE"))
    local_rank = int(os.getenv("RANK_ID"))
    if device_num == 1:
        obs_copy_folder(train_dir, train_url)
    if device_num > 1:
        if local_rank % 8 == 0:
            obs_copy_folder(train_dir, train_url)
    return


def obs_copy_file(obs_file_url, file_url):
    """
    cope file from obs to obs, or cope file from obs to env, or cope file from env to obs
    """
    try:
        mox.file.copy(obs_file_url, file_url)
        print("Successfully Download {} to {}".format(obs_file_url, file_url))
    except Exception as e:
        print(
            "moxing download {} to {} failed: ".format(obs_file_url, file_url) + str(e)
        )
    return


def obs_copy_folder(folder_dir, obs_folder_url):
    """
    copy folder from obs to obs, or copy folder from obs to env, or copy folder from env to obs
    """
    try:
        mox.file.copy_parallel(folder_dir, obs_folder_url)
        print("Successfully Upload {} to {}".format(folder_dir, obs_folder_url))
    except Exception as e:
        print(
            "moxing upload {} to {} failed: ".format(folder_dir, obs_folder_url)
            + str(e)
        )
    return


def c2net_multidataset_to_env(multi_data_url, data_dir):
    """
    c2net copy single or multi dataset to training image
    """
    multi_data_json = json.loads(multi_data_url)
    for i in range(len(multi_data_json)):
        zipfile_path = data_dir + "/" + multi_data_json[i]["dataset_name"]
        try:
            mox.file.copy(multi_data_json[i]["dataset_url"], zipfile_path)
            print(
                "Successfully Download {} to {}".format(
                    multi_data_json[i]["dataset_url"], zipfile_path
                )
            )
            # get filename and unzip the dataset
            filename = os.path.splitext(multi_data_json[i]["dataset_name"])[0]
            filePath = data_dir + "/" + filename
            if not os.path.exists(filePath):
                os.makedirs(filePath)
            if zipfile_path.endswith(".tar.gz"):
                os.system("tar -zxvf {} -C {}".format(zipfile_path, filePath))
            elif zipfile_path.endswith(".zip"):
                os.system("unzip {} -d {}".format(zipfile_path, filePath))
            else:
                print("The dataset is not in tar.gz or zip format!")

        except Exception as e:
            print(
                "moxing download {} to {} failed: ".format(
                    multi_data_json[i]["dataset_url"], zipfile_path
                )
                + str(e)
            )
    return


class EnvToOpenIEpochEnd(Callback):
    """
    upload output to openi when epoch end
    """

    def __init__(self, train_dir, obs_train_url):
        self.train_dir = train_dir
        self.obs_train_url = obs_train_url

    def epoch_end(self, run_context):
        obs_copy_folder(self.train_dir, self.obs_train_url)



import os


def get_code_path_minio():
    """
    获取源代码存储在minio,挂载到镜像后的代码路径
    """
    code_path = os.getenv("CODE_PATH")
    if code_path is None:
        raise ValueError(
            "Failed to get the environment variable, please ensure that the CODE_PATH environment variable has been set."
        )
    return code_path


def get_data_path_minio():
    """
    获取源数据集存储在minio,挂载到镜像后的数据集路径
    """
    data_path = os.getenv("DATA_PATH")
    if data_path is None:
        raise ValueError(
            "Failed to get the environment variable, please ensure that the DATA_PATH environment variable has been set."
        )
    return data_path


def get_pretrain_model_path_minio():
    """
    获取源预训练模型存储在minio,挂载到镜像后的预训练模型路径
    """
    pretrain_model_path = os.getenv("PRETRAIN_MODEL_PATH")
    if pretrain_model_path is None:
        raise ValueError(
            "Failed to get the environment variable, please make sure the PRETRAIN_MODEL_PATH environment variable has been set."
        )
    return pretrain_model_path


def get_output_path_minio():
    """
    获取需要存储在minio的输出路径
    """
    output_path = os.getenv("OUTPUT_PATH")
    if output_path is None:
        raise ValueError(
            "Failed to get the environment variable, please ensure that the OUTPUT_PATH environment variable has been set."
        )
    return output_path



from .dataset_file import *
from openi.apis import OpeniAPI
from openi.utils import file_utils, logger
from openi.settings import *
from tqdm import tqdm
import logging

logger.setup_logger()


def upload_with_tqdm(_file: DatasetFile, api: OpeniAPI, upload_chunks: list):
    chunk_size = _file.chunk_size
    _progress = chunk_size * (_file.total_chunks_count - len(upload_chunks))

    with tqdm(
        total=_file.size,
        leave=True,
        initial=_progress,
        unit="B",
        unit_scale=True,
        unit_divisor=1000,
        bar_format="{desc}{percentage:3.0f}%|{bar}{r_bar}",
        desc=f"{logger.get_time()} - Uploading: ",
        dynamic_ncols=True,
    ) as pbar:
        msg = f"start uploading with progress bar, file object {_file.__dict__}"
        logging.info(msg)

        # upload_chunks
        for n in upload_chunks:
            start_position = (n - 1) * chunk_size
            real_chunk_size = min(chunk_size, _file.size - start_position)

            data = file_utils.read_file_chunk(
                _file.filepath, start_position, real_chunk_size
            )
            for i in range(real_chunk_size):
                pbar.update(1)

            url = api.get_multipart_url(n, chunk_size, _file.__dict__)
            etag = api.put_upload(url, data, _file.upload_type)
            if etag is None:
                msg = (
                    f"❌ Upload failed: {_file.filename}({_file.cluster}) "
                    f"chunk {n} failed to upload."
                )
                logging.error(msg)
                raise RuntimeError(msg)


def upload_file(
    file: str,
    repo_id: str,
    cluster: str = "NPU",
    endpoint: str = None,
    token: str = None,
):
    api = OpeniAPI(endpoint=endpoint, token=token)

    _dataset_info = api.get_dataset(repo_id=repo_id)
    if _dataset_info == []:
        msg = f"❌ `{repo_id}` no dataset was found, please create a dataset first before upload files."
        logging.error(msg)
        raise ValueError(msg)

    # file object init
    _file = DatasetFile(file=file, repo_id=repo_id, cluster=cluster.upper())
    _file.upload_size_check()
    _file.dataset_id = _dataset_info[0]["id"]

    msg = (
        f"{logger.get_time()} - `{_file.filename}`({_file.cluster}) calculating md5..."
    )
    logging.info(msg)
    print(msg)
    try:
        _file.md5 = file_utils.calculateMD5(_file.filepath)
    except Exception as e:
        logging.error(e)
        raise ValueError(e)

    _get_chunks = api.get_chunks(_file.__dict__)

    # upload new
    if _get_chunks["uuid"] == "" or _get_chunks["uploadID"] == "":
        _file.uuid, _file.upload_id = api.new_multipart(_file.__dict__)
        if _get_chunks["uuid"] == "" and _get_chunks["uploadID"] == "":
            upload_chunks = [i for i in range(1, _file.total_chunks_count + 1)]
            upload_with_tqdm(_file, api, upload_chunks)
        else:
            msg = f"❌ Upload failed: {_file.filename}` ({cluster}), please contact us. "
            logging.error(msg)
            raise RuntimeError(msg)

    # upload completed
    if _get_chunks["uploaded"] == "1":
        if _get_chunks["datasetID"] != "" and _get_chunks["datasetName"] != "":
            msg = (
                f"❌ Upload failed: `{_file.filename}` ({cluster})"
                " already exists in "
                f"{api.endpoint.split('/api')[0]}/{_get_chunks['repoOwner']}/{_get_chunks['repoName']}/datasets"
            )
            logging.error(msg)
            raise ValueError(msg)

    # upload continue
    if _get_chunks["uuid"] != "" or _get_chunks["uploadID"] != "":
        _file.uuid, _file.upload_id = _get_chunks["uuid"], _get_chunks["uploadID"]
        continue_index = max(
            [int(i.split("-")[0]) for i in _get_chunks["chunks"].split(",") if i != ""]
        )
        continue_chunks = [
            i for i in range(continue_index, _file.total_chunks_count + 1)
        ]
        upload_with_tqdm(_file, api, continue_chunks)

    # complete upload process
    api.complete_multipart(_file.__dict__)
    _url = f"{api.endpoint.split('/api')[0]}/{repo_id}/datasets"
    msg = f"{logger.get_time()} - 🎉 Successfully uploaded, view on link: {_url}"
    logging.info(msg)
    print(msg)



import os
from .env_check import (
    openi_multidataset_to_env,
    c2net_multidataset_to_env,
    pretrain_to_env,
    env_to_openi,
)


# 需要定义传给modelarts的两个参数data_url和train_url或是使用args, unknown = parser.parse_known_args()来规避超参数没定义的报错
def get_code_path_obs():
    """
    获取源数据代码存储在obs,拷贝到镜像后的代码路径
    """
    code_url = os.getenv("CODE_URL")
    code_path = os.getenv("CODE_PATH")
    if code_url is None or code_path is None:
        raise ValueError(
            "Failed to get environment variables, please make sure you have set PRETRAIN_MODEL_URL, PRETRAIN_MODEL_PATH environment variables."
        )
    env_to_openi(code_url, code_path)
    return code_path


def get_data_path_obs():
    """
    获取源数据集存储在obs,拷贝到镜像后的数据集路径
    """
    cluster = os.getenv("CLUSTER")
    data_url = os.getenv("DATA_URL")
    data_path = os.getenv("DATA_PATH")
    if cluster is None or data_url is None or data_path is None:
        raise ValueError(
            "Failed to get the environment variable, please make sure the CLUSTER, DATA_URL and DATA_PATH environment variables have been set"
        )
    if cluster == "c2net":
        c2net_multidataset_to_env(data_url, data_path)
    else:
        openi_multidataset_to_env(data_url, data_path)
    return data_path


def get_pretrain_model_path_obs():
    """
    获取源数据预训练模型存储在obs,拷贝到镜像后的预训练模型路径
    """
    pretrain_model_url = os.getenv("PRETRAIN_MODEL_URL")
    pretrain_model_path = os.getenv("PRETRAIN_MODEL_PATH")
    if pretrain_model_url is None or pretrain_model_path is None:
        raise ValueError(
            "Failed to get environment variables, please make sure you have set PRETRAIN_MODEL_URL, PRETRAIN_MODEL_PATH environment variables."
        )
    pretrain_to_env(pretrain_model_url, pretrain_model_path)
    return pretrain_model_path


def get_output_path_obs():
    """
    获取需要存储在obs上的输出路径
    """
    output_path = os.getenv("OUTPUT_PATH")
    if output_path is None:
        raise ValueError(
            "Failed to get the environment variable, please ensure that the OUTPUT_PATH environment variable has been set."
        )
    return output_path


def push_output_to_openi_obs():
    """
    将输出数据推送到openi的obs存储
    """
    output_url = os.getenv("OUTPUT_URL")
    output_path = os.getenv("OUTPUT_PATH")
    if output_url is None or output_path is None:
        raise ValueError(
            "Failed to get environment variables, please ensure that the OUTPUT_URL and OUTPUT_PATH environment variables have been set."
        )
    env_to_openi(OUTPUT_PATH, OUTPUT_URL)
    return



import logging
import os
from ..settings import *
from datetime import datetime


def setup_logger():
    if not os.path.exists(PATH.OPENI_FOLDER):
        os.mkdir(PATH.OPENI_FOLDER)
    LOG_FORMAT = "%(asctime)s [%(levelname)s] - %(filename)s %(funcName)s() %(message)s"
    DATE_FORMAT = "%Y/%m/%d %H:%M:%S"
    logging.basicConfig(
        filename=PATH.LOG_PATH,
        level=logging.INFO,
        format=LOG_FORMAT,
        datefmt=DATE_FORMAT,
    )


def get_time():
    return datetime.now().strftime("%H:%M:%S")


def debug(ob):
    if type(ob) == dict:
        import json

        print(json.dumps(ob, indent=4))
    else:
        print(ob)
    print("Debug stop!")
    exit(1)



import hashlib
import math
import json
from datetime import datetime
from pathlib import Path
from openi.settings import *


def calculateMD5(filepath: str = None) -> str:
    """
    计算文件的完整md5
    :param self.filepath:
    :return:
    """
    m = hashlib.md5()  # 创建md5对象
    with open(filepath, "rb") as f:
        while True:
            data = f.read(4096)
            if not data:
                break
            m.update(data)  # 更新md5对象
    return m.hexdigest()  # 返回md5对象


def read_file_chunk(
    filepath: str = None, start_position: int = 0, chunk_size: int = None
):
    with open(filepath, "rb") as file:
        file.seek(start_position)
        chunk = file.read(chunk_size)
        return chunk


def get_file_chunk(chunk_size: int, filesize: int = 0) -> dict:
    total_chunks_count = math.ceil(filesize / chunk_size)
    return total_chunks_count


def get_token() -> str:
    return json.loads(Path(PATH.TOKEN_PATH).read_text())


def rename_existing_file(filepath):
    path, suffix = os.path.splitext(filepath)
    counter = 0
    filename = "{}({}){}"
    while os.path.exists(filename.format(path, counter, suffix)):
        counter += 1
    new = filename.format(path, counter, suffix)

    return new



import os
from ..cloudbrain.obs_operate import (
    get_code_path_obs,
    get_data_path_obs,
    get_pretrain_model_path_obs,
    get_output_path_obs,
    push_output_to_openi_obs,
)
from ..cloudbrain.minio_operate import (
    get_code_path_minio,
    get_data_path_minio,
    get_pretrain_model_path_minio,
    get_output_path_minio,
)


def get_code_path():
    """
    获取代码路径
    """
    if os.getenv("STORAGE_LOCATION") is None:
        raise ValueError(
            "Failed to get the environment variable, please ensure that the STORAGE_LOCATION environment variable has been set."
        )
    if os.getenv("STORAGE_LOCATION") == "obs":
        return get_code_path_obs()
    return get_code_path_minio()


def get_data_path():
    """
    获取数据集路径
    """
    if os.getenv("STORAGE_LOCATION") is None:
        raise ValueError(
            "Failed to get the environment variable, please make sure the STORAGE_LOCATION environment variable has been set"
        )
    if os.getenv("STORAGE_LOCATION") == "obs":
        return get_data_path_obs()
    return get_data_path_minio()


def get_pretrain_model_path():
    """
    获取预训练模型路径
    """
    if os.getenv("STORAGE_LOCATION") is None:
        raise ValueError(
            "Failed to get the environment variable, please make sure the STORAGE_LOCATION environment variable has been set."
        )
    if os.getenv("STORAGE_LOCATION") == "obs":
        return get_pretrain_model_path_obs()
    return get_pretrain_model_path_minio()


def get_output_path():
    """
    获取输出路径
    """
    if os.getenv("STORAGE_LOCATION") is None:
        raise ValueError(
            "Failed to get the environment variable, please make sure the STORAGE_LOCATION environment variable has been set."
        )
    if os.getenv("STORAGE_LOCATION") == "obs":
        return get_output_path_obs()
    return get_output_path_minio()


def push_output_to_openi():
    """
    推送输出结果到启智平台
    """
    if os.getenv("STORAGE_LOCATION") is None:
        raise ValueError(
            "Failed to get the environment variable, please make sure the STORAGE_LOCATION environment variable has been set."
        )
    if os.getenv("STORAGE_LOCATION") == "obs":
        return push_output_to_openi_obs()
    return



import os
import json
from getpass import getpass
from pathlib import Path
from openi.apis import OpeniAPI
from .settings import *
from openi.utils.file_utils import get_token
from openi.utils import logger
import logging

logger.setup_logger()


def login(token: str = None, endpoint: str = API.ENDPOINT) -> None:
    print(
        """\n
             ██████╗   ██████╗  ███████╗  ███╗   ██╗  ██████╗
            ██╔═══██╗  ██╔══██╗ ██╔════╝  ████╗  ██║    ██╔═╝
            ██║   ██║  ██████╔╝ █████╗    ██╔██╗ ██║    ██║
            ██║   ██║  ██╔═══╝  ██╔══╝    ██║╚██╗██║    ██║
            ╚██████╔╝  ██║      ███████╗  ██║ ╚████║  ██████╗
             ╚═════╝   ╚═╝      ╚══════╝  ╚═╝  ╚═══╝  ╚═════╝\n
         """
    )
    endpoint = endpoint[:-1] if endpoint[-1] == "/" else endpoint

    if token is None:
        print(f"点击链接获取令牌并复制粘贴到下列输入栏 {endpoint}/user/settings/applications \n")
        print(
            f"[WARNING] 若本机已存在登录令牌，本次输入的令牌会将其覆盖 \n"
            "          若粘贴时切换了本窗口，请先按 退格键⇦ 删除多余空格\n"
        )
        token = getpass(prompt="  🔒 token: ")
    _login(token=token, endpoint=endpoint)


def _login(token: str, endpoint: str) -> None:
    try:
        api = OpeniAPI(endpoint, token)
        valid_user = api.login_check()

        if not os.path.exists(PATH.OPENI_FOLDER):
            os.mkdir(PATH.OPENI_FOLDER)
        Path(PATH.TOKEN_PATH).write_text(
            json.dumps({"endpoint": endpoint, "token": token, "username": valid_user})
        )
        msg = (
            f"\n  Your token was saved to `{PATH.TOKEN_PATH}`\n"
            f"  Successfully logged in as `{valid_user}` @{endpoint}!\n"
        )
        logging.info(msg)
        print(msg)
    except:
        raise ValueError("\n  ❌ login failed: invalid token or endpoint!\n")


def whoami() -> None:
    try:
        api = OpeniAPI()
        valid_user = api.login_check()
        msg = f"\n`{valid_user}` @{api.baseURL}\n"
        logging.info(msg)
        print(msg)
    except:
        msg = f"\nCurrently not logged in.\n"
        logging.info(msg)
        print(msg)


def logout() -> None:
    try:
        valid_user = get_token()["username"]
        os.remove(PATH.TOKEN_PATH)
        msg = f"\n`{valid_user}` successfully logged out.\n"
        logging.info(msg)
        print(msg)
    except:
        msg = "\nCurrently not logged in.\n"
        logging.info(msg)
        print(msg)



import logging

import requests
from openi.settings import *
from openi.utils import file_utils

"""
APIS
"""


class OpeniAPI:
    def __init__(self, endpoint: str = None, token: str = None):
        self.baseURL = (
            file_utils.get_token()["endpoint"] if endpoint is None else endpoint
        )
        self.endpoint = self.baseURL + API.VERSION
        self.token = file_utils.get_token()["token"] if token is None else token

    def catch_auth_error(self, x):
        if x.status_code == 401:
            msg = f"❌ {x} {x.reason} - failed to connecting OPENI, please check your `token` or permssion."
            logging.error(msg)
            raise PermissionError(msg)
        if x.status_code == 404:
            msg = f"❌ {x} {x.reason} - failed to connecting OPENI, either incorrect `owner/repo` or `endpoint`."
            logging.error(msg)
            raise ConnectionError(msg)
        if x.raise_for_status():
            logging.error(x.raise_for_status())
            x.raise_for_status()

    def login_check(self) -> str:
        params = {"access_token": self.token}
        x = requests.get(f"{self.endpoint}/user", params=params)
        logging.info(f"GET {x.url}")
        self.catch_auth_error(x)
        logging.info(f"{x} {x.url} {x.json()}")
        return x.json()["username"]

    def get_dataset(self, repo_id):
        params = {"access_token": self.token}
        x = requests.get(f"{self.endpoint}/datasets/{repo_id}", params=params)
        logging.info(f"GET {x.url}")
        self.catch_auth_error(x)
        logging.info(f"{x} {x.url} {x.json()}")
        return x.json()["data"]

    def get_dataset_attachment(self, repo_id: str, upload_type: str):
        params = {"access_token": self.token, "type": upload_type}
        x = requests.get(
            f"{self.endpoint}/datasets/{repo_id}/current_repo", params=params
        )
        logging.info(f"GET {x.url}")
        self.catch_auth_error(x)
        logging.info(f"{x} {x.url} {x.json()}")
        return x.json()

    def get_chunks(self, _file):
        params = {
            "access_token": self.token,
            "dataset_id": _file["dataset_id"],
            "md5": _file["md5"],
            "file_name": _file["filename"],
            "type": _file["upload_type"],
        }
        x = requests.get(f"{self.endpoint}/attachments/get_chunks", params=params)
        logging.info(f"GET {x.url}")
        if x.ok:
            logging.info(f"{x} {x.url} {x.json()}")
            return x.json()
        else:
            msg = f"❌ {x} {x.reason} {x.text}"
            logging.error(msg)
            raise ConnectionRefusedError(msg)

    def get_multipart_url(self, chunk_number: int, chunk_size: int, _file: dict):
        params = {
            "access_token": self.token,
            "dataset_id": _file["dataset_id"],
            "file_name": _file["filename"],
            "type": _file["upload_type"],
            "chunkNumber": chunk_number,
            "size": chunk_size,
            "uploadID": _file["upload_id"],
            "uuid": _file["uuid"],
        }
        x = requests.get(
            f"{self.endpoint}/attachments/get_multipart_url", params=params
        )
        logging.info(f"GET {x.url}")
        if x.ok:
            logging.info(f"{x} {x.url} {x.json()}")
            return x.json()["url"]
        else:
            msg = f"❌ {x} {x.reason}"
            logging.error(msg)
            raise ConnectionRefusedError(msg)

    def put_upload(self, url, data, upload_type):
        headers = {"Content-Type": "text/plain"} if upload_type == 0 else {}
        x = requests.put(url, data=data, headers=headers)
        logging.info(f"PUT {x.url}")
        try:
            logging.info(f"{x} {x.url} {x.headers}")
            return x.headers["ETag"]
        except:
            msg = f"❌ {x} {x.reason} {x.text}"
            logging.error(msg)
            raise ConnectionRefusedError(msg)

    def new_multipart(self, _file):
        params = {
            "access_token": self.token,
            "dataset_id": _file["dataset_id"],
            "md5": _file["md5"],
            "file_name": _file["filename"],
            "type": _file["upload_type"],
            "totalChunkCounts": _file["total_chunks_count"],
            "size": _file["size"],
        }
        x = requests.get(f"{self.endpoint}/attachments/new_multipart", params=params)
        logging.info(f"GET {x.url}")
        if x.ok:
            if x.json()["result_code"] == "0":
                logging.info(f"{x} {x.url} {x.json()}")
                return x.json()["uuid"], x.json()["uploadID"]
            else:
                logging.error(f"❌ {x} {x.reason} {x.json()}")
                raise ConnectionRefusedError(f'❌ {x} {x.reason} {x.json()["msg"]}')
        else:
            msg = f"❌ {x} {x.reason}"
            logging.error(msg)
            raise ConnectionRefusedError(msg)

    def complete_multipart(self, _file):
        params = {
            "access_token": self.token,
            "dataset_id": _file["dataset_id"],
            "file_name": _file["filename"],
            "type": _file["upload_type"],
            "size": _file["size"],
            "uploadID": _file["upload_id"],
            "uuid": _file["uuid"],
        }
        x = requests.post(
            f"{self.endpoint}/attachments/complete_multipart", params=params
        )
        logging.info(f"POST {x.url}")
        if x.ok:
            if x.json()["result_code"] == "-1":
                logging.error(f"❌ {x} {x.reason} {x.json()}")
                raise ConnectionRefusedError(f'❌ {x} {x.reason} {x.json()["msg"]}')
            else:
                logging.info(f"{x} {x.url} {x.json()}")
        else:
            msg = f"❌ {x} {x.reason} {x.text}"
            logging.error(msg)
            raise ConnectionRefusedError(msg)

    def download_attachments(self, uuid: str, upload_type: int):
        params = {"access_token": self.token, "type": upload_type}
        x = requests.get(
            f"{self.endpoint}/attachments/{uuid}",
            params=params,
            allow_redirects=True,
            stream=True,
        )
        logging.info(f"GET {x.url}")
        self.catch_auth_error(x)
        logging.info(f"{x} {x.url}")
        return x



import emoji


def env_check():
    try:
        import moxing as mox

        print(
            f'{emoji.emojize(":thumbs_up:")} enviornment check pass: Modelarts enviornment checked'
        )
        return True
    except:
        print(
            f'{emoji.emojize(":cross_mark:")} enviornment check failed: 请在启智平台NPU集群上运行该代码'
        )
        return False


def openi_dataset_to_env(data_url, data_dir):
    if env_check():
        from .helper import openi_dataset_to_env as func

        func(data_url, data_dir)


def openi_multidataset_to_env(multi_data_url, data_dir):
    if env_check():
        from .helper import openi_multidataset_to_env as func

        func(multi_data_url, data_dir)


def pretrain_to_env(pretrain_url, pretrain_dir):
    if env_check():
        from .helper import pretrain_to_env as func

        func(pretrain_url, pretrain_dir)


def env_to_openi(train_dir, train_url):
    if env_check():
        from .helper import env_to_openi as func

        func(train_dir, train_url)


def obs_copy_file(obs_file_url, file_url):
    if env_check():
        from .helper import obs_copy_file as func

        func(obs_file_url, file_url)


def obs_copy_folder(folder_dir, obs_folder_url):
    if env_check():
        from .helper import obs_copy_folder as func

        func(folder_dir, obs_folder_url)


def c2net_multidataset_to_env(multi_data_url, data_dir):
    if env_check():
        from .helper import c2net_multidataset_to_env as func

        func(multi_data_url, data_dir)
